@@ -75,15 +75,13 @@ namespace std {
         template<class _Dom>
         void operator>>= (const _Expr<_Dom,_Tp>&) const;
         
-        //        ~gslice_array();
-
     private:
         _Array<_Tp>    _M_array;
-        const gslice& _M_gs;
+        const valarray<size_t>& _M_index;
         
         friend class valarray<_Tp>;
         
-        gslice_array (_Array<_Tp>, const gslice& );
+        gslice_array (_Array<_Tp>, const valarray<size_t>&);
 
         // this constructor needs to be implemented.
         gslice_array (const gslice_array&);
@@ -94,23 +92,24 @@ namespace std {
     };
 
     template<typename _Tp>
-    inline gslice_array<_Tp>::gslice_array (_Array<_Tp> __a, const gslice& __g)
-            : _M_array (__a), _M_gs (__g) {}
+    inline
+    gslice_array<_Tp>::gslice_array (_Array<_Tp> __a,
+                                     const valarray<size_t>& __i)
+            : _M_array (__a), _M_index (__i) {}
 
 
     template<typename _Tp>
-    inline gslice_array<_Tp>::gslice_array (const gslice_array<_Tp>& __a)
-            : _M_array (__a._M_array), _M_gs (__a._M_gs) {}
+    inline
+    gslice_array<_Tp>::gslice_array (const gslice_array<_Tp>& __a)
+            : _M_array (__a._M_array), _M_index (__a._M_index) {}
 
     
-    // template<typename _Tp>
-    // inline gslice_array<_Tp>::~gslice_array () {}
-
     template<typename _Tp>
     inline void
     gslice_array<_Tp>::operator= (const _Tp& __t) 
     { 
-        __valarray_fill (_M_array, _M_gs._M_index, _M_gs._M_index_size, __t); 
+        __valarray_fill (_M_array, _Array<size_t>(_M_index),
+                         _M_index.size(), __t); 
     }
 
     template<typename _Tp>
@@ -118,7 +117,7 @@ namespace std {
     gslice_array<_Tp>::operator= (const valarray<_Tp>& __v) const
     {
         __valarray_copy (_Array<_Tp> (__v), __v.size (),
-                         _M_array, _M_gs._M_index);
+                         _M_array, _Array<size_t>(_M_index));
     }
 
     template<typename _Tp>
@@ -126,7 +125,8 @@ namespace std {
     inline void
     gslice_array<_Tp>::operator= (const _Expr<E, _Tp>& __e) const
     {
-        __valarray_copy (__e, _M_gs._M_index_size, _M_array, _M_gs._M_index);
+        __valarray_copy (__e, _M_index.size(), _M_array,
+                         _Array<size_t>(_M_index));
     }
 
 #undef _DEFINE_VALARRAY_OPERATOR
@@ -135,7 +135,7 @@ template<typename _Tp>							\
 inline void								\
 gslice_array<_Tp>::operator##op##= (const valarray<_Tp>& __v) const	\
 {									\
-    _Array_augmented_##name (_M_array, _M_gs._M_index,			\
+    _Array_augmented_##name (_M_array, _Array<size_t>(_M_index),	\
                               _Array<_Tp> (__v), __v.size ());		\
 }									\
 									\
@@ -143,8 +143,8 @@ template<typename _Tp> template<class E>        			\
 inline void 								\
 gslice_array<_Tp>::operator##op##= (const _Expr<E, _Tp>& __e) const	\
 {									\
-    _Array_augmented_##name (_M_array, _M_gs._M_index, __e,		\
-                              _M_gs._M_index_size);			\
+    _Array_augmented_##name (_M_array, _Array<size_t>(_M_index), __e,	\
+                              _M_index.size());				\
 }
 
 _DEFINE_VALARRAY_OPERATOR(*, multiplies)
