@@ -112,9 +112,8 @@ namespace std
 
 
   // 22.2.1.1  Template class ctype
-  // _Ctype_nois is the common base for ctype<char> and ctype<T>.  It
-  // lacks "do_is" and related virtuals.  These are filled in by
-  // _Ctype, below.
+  // _Ctype_nois is the common base for ctype<char>.  It lacks "do_is"
+  // and related virtuals.  These are filled in by _Ctype, below.
   template<typename _CharT>
     class _Ctype_nois : public locale::facet, public ctype_base
     {
@@ -124,40 +123,40 @@ namespace std
     public:
       char_type 
       toupper(char_type __c) const
-      { return do_toupper(__c); }
+      { return this->do_toupper(__c); }
 
       const char_type*
       toupper(char_type *__low, const char_type* __high) const
-      { return do_toupper(__low, __high); }
+      { return this->do_toupper(__low, __high); }
 
       char_type 
       tolower(char_type __c) const
-      { return do_tolower(__c); }
+      { return this->do_tolower(__c); }
 
       const char_type*
       tolower(char_type* __low, const char_type* __high) const
-      { return do_tolower(__low, __high); }
+      { return this->do_tolower(__low, __high); }
 
       char_type 
       widen(char __c) const
-      { return do_widen(__c); }
+      { return this->do_widen(__c); }
 
       const char*
       widen(const char* __low, const char* __high, char_type* __to) const
-      { return do_widen(__low, __high, __to); }
+      { return this->do_widen(__low, __high, __to); }
 
       char 
       narrow(char_type __c, char __dfault) const
-      { return do_narrow(__c, __dfault); }
+      { return this->do_narrow(__c, __dfault); }
 
       const char_type*
       narrow(const char_type* __low, const char_type* __high,
 	      char __dfault, char *__to) const
-      { return do_narrow(__low, __high, __dfault, __to); }
+      { return this->do_narrow(__low, __high, __dfault, __to); }
 
     protected:
       explicit 
-      _Ctype_nois(size_t __refs = 0) : locale::facet(__refs) { }
+      _Ctype_nois(size_t __refs = 0): locale::facet(__refs) { }
 
       virtual 
       ~_Ctype_nois() { }
@@ -195,27 +194,30 @@ namespace std
     {
       // Types:
       typedef _CharT char_type;
+
     public:
       bool 
       is(mask __m, char_type __c) const
-      { return do_is(__m, __c); }
+      { return this->do_is(__m, __c); }
 
       const char_type*
       is(const char_type *__lo, const char_type *__hi, mask *__vec) const   
-      { return do_is(__lo, __hi, __vec); }
+      { return this->do_is(__lo, __hi, __vec); }
 
       const char_type*
       scan_is(mask __m, const char_type* __lo, const char_type* __hi) const
-      { return do_scan_is(__m, __lo, __hi); }
+      { return this->do_scan_is(__m, __lo, __hi); }
 
       const char_type*
       scan_not(mask __m, const char_type* __lo, const char_type* __hi) const
-      { return do_scan_not(__m, __lo, __hi); }
+      { return this->do_scan_not(__m, __lo, __hi); }
+
     protected:
       explicit 
       _Ctype(size_t __refs = 0) : _Ctype_nois<_CharT>(__refs) { }
 
-      virtual ~_Ctype() { }
+      virtual 
+      ~_Ctype() { }
 
       virtual bool 
       do_is(mask __m, char_type __c) const = 0;
@@ -241,18 +243,97 @@ namespace std
 
     public:
       explicit 
-      ctype(size_t __refs = 0) : _Ctype<_CharT> (__refs) { }
+      ctype(size_t __refs = 0) : _Ctype<_CharT>(__refs) { }
 
       static locale::id id;
 
    protected:
       virtual 
       ~ctype() { }
-      // Other members inherited.
+
+      virtual bool 
+      do_is(mask __m, char_type __c) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_is(const char_type* __lo, const char_type* __hi, 
+	    mask* __vec) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_scan_is(mask __m, const char_type* __lo, 
+		 const char_type* __hi) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_scan_not(mask __m, const char_type* __lo, 
+		  const char_type* __hi) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual char_type 
+      do_toupper(char_type) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_toupper(char_type* __low, const char_type* __high) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual char_type 
+      do_tolower(char_type) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_tolower(char_type* __low, const char_type* __high) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+      
+      virtual char_type 
+      do_widen(char) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char*
+      do_widen(const char* __low, const char* __high,
+	       char_type* __dest) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual char 
+      do_narrow(char_type, char __dfault) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
+
+      virtual const char_type*
+      do_narrow(const char_type* __low, const char_type* __high,
+		 char __dfault, char* __dest) const
+      {
+	// XXX Need definitions for these abstract mf's.
+      }
     };
 
 
   // 22.2.1.3  ctype specializations
+  // NB: Can use _Ctype_nois to actually implement the is
+  // functionality in the non-virtual (thus inline-able) member
+  // fuctions.
   template<>
     class ctype<char> : public _Ctype_nois<char>
     {
@@ -305,40 +386,7 @@ namespace std
 	  ++__low;
 	return __low;
       }
-
-      char_type 
-      toupper(char_type __c) const
-      { return do_toupper(__c); }
-
-      const char_type*
-      toupper(char_type *__low, const char_type* __high) const
-      { return do_toupper(__low, __high); }
-
-      char_type
-      tolower(char_type __c) const
-      { return do_tolower(__c); }
-
-      const char_type*
-      tolower(char_type* __low, const char_type* __high) const
-      { return do_tolower(__low, __high); }
-
-      char_type
-      widen(char __c) const
-      { return do_widen(__c); }
-
-      const char*
-      widen(const char* __low, const char* __high, char_type* __to) const
-      { return do_widen(__low, __high, __to); }
-
-      char
-      narrow(char_type __c, char __dfault) const
-      { return do_narrow(__c, __dfault); }
-
-      const char_type*
-      narrow(const char_type* __low, const char_type* __high,
-	     char __dfault, char *__to) const
-      { return do_narrow(__low, __high, __dfault, __to); }
-      
+     
     protected:
       virtual 
       ~ctype();
@@ -351,30 +399,31 @@ namespace std
       classic_table() throw()
       { return _S_table; }
 
-      virtual char        
-      do_toupper(char) const;
+      virtual char_type 
+      do_toupper(char_type) const;
 
-      virtual const char* 
-      do_toupper(char* __low, const char* __high) const;
-      
-      virtual char        
-      do_tolower(char) const;
+      virtual const char_type*
+      do_toupper(char_type* __low, const char_type* __high) const;
 
-      virtual const char* 
-      do_tolower(char* __low, const char* __high) const;
+      virtual char_type 
+      do_tolower(char_type) const;
+
+      virtual const char_type*
+      do_tolower(char_type* __low, const char_type* __high) const;
       
-      virtual char        
+      virtual char_type 
       do_widen(char) const;
 
-      virtual const char* 
-      do_widen(const char* __low, const char* __high, char* __dest) const;
+      virtual const char*
+      do_widen(const char* __low, const char* __high,
+	       char_type* __dest) const;
 
-      virtual char        
-      do_narrow(char, char __dfault) const;
+      virtual char 
+      do_narrow(char_type, char __dfault) const;
 
-      virtual const char* 
-      do_narrow(const char* __low, const char* __high,
-		char __dfault, char* __dest) const;
+      virtual const char_type*
+      do_narrow(const char_type* __low, const char_type* __high,
+		 char __dfault, char* __dest) const;
     };
 
   template<>
@@ -383,13 +432,14 @@ namespace std
 
 #ifdef _GLIBCPP_USE_WCHAR_T
   // ctype<wchar_t> specialization
+  // XXX Not done, need missing member functions.
   template<>
     class ctype<wchar_t> : public _Ctype<wchar_t>
     {
       // Types:
       typedef wchar_t char_type;
     public:
-      explicit ctype (size_t __refs = 0);
+      explicit ctype(size_t __refs = 0);
       static locale::id id;
 
     private:
@@ -399,40 +449,50 @@ namespace std
       static const int _S_table_size = ctype<char>::table_size;
       
     protected:
-      virtual ~ctype ();
+      virtual 
+      ~ctype();
+
+      virtual bool 
+      do_is(mask __m, char_type __c) const;
+
+      virtual const char_type*
+      do_is(const char_type* __lo, const char_type* __hi, 
+	    mask* __vec) const;
+
+      virtual const char_type*
+      do_scan_is(mask __m, const char_type* __lo, 
+		 const char_type* __hi) const;
+
+      virtual const char_type*
+      do_scan_not(mask __m, const char_type* __lo, 
+		  const char_type* __hi) const;
+
+      virtual char_type 
+      do_toupper(char_type) const;
+
+      virtual const char_type*
+      do_toupper(char_type* __low, const char_type* __high) const;
+
+      virtual char_type 
+      do_tolower(char_type) const;
+
+      virtual const char_type*
+      do_tolower(char_type* __low, const char_type* __high) const;
       
-      virtual bool
-      do_is (mask __m, char_type __c) const;
-      virtual const char_type*
-      do_is (const char_type* __low,
-	     const char_type* __high, mask* __vec) const;
+      virtual char_type 
+      do_widen(char) const;
 
-      virtual const char_type*
-      do_scan_is (mask __m, const char_type* __low,
-		  const char_type* __high) const;
-      virtual const char_type*
-      do_scan_not (mask __m, const char_type* __low,
-		   const char_type* __high) const;
-
-      virtual char_type
-      do_toupper (char_type) const;
-      virtual const char_type*
-      do_toupper (char_type* __low, const char_type* __high) const;
-      virtual char_type
-      do_tolower (char_type) const;
-      virtual const char_type*
-      do_tolower (char_type* __low, const char_type* __high) const;
-
-      virtual char_type
-      do_widen (char) const;
       virtual const char*
-      do_widen (const char* __low, const char* __high,
-		char_type* __dest) const;
-      virtual char
-      do_narrow (char_type, char __dfault) const;
+      do_widen(const char* __low, const char* __high,
+	       char_type* __dest) const;
+
+      virtual char 
+      do_narrow(char_type, char __dfault) const;
+
       virtual const char_type*
-      do_narrow (const char_type* __low, const char_type* __high,
+      do_narrow(const char_type* __low, const char_type* __high,
 		 char __dfault, char* __dest) const;
+
     };
 
   template<>
@@ -462,6 +522,7 @@ namespace std
     ctype_byname<wchar_t>::ctype_byname(const char*, size_t refs);
 #endif
 
+
   //  22.2.1.5  Template class codecvt
   class codecvt_base
   {
@@ -939,7 +1000,8 @@ namespace std
       // functions. NB: this is specialized for char.
       void 
       _M_extract(iter_type __beg, iter_type __end, ios_base& __io, 
-		 ios_base::iostate& __err, char* __xtrc, int& __base) const;
+		 ios_base::iostate& __err, char* __xtrc, 
+		 int& __base, bool __fp = true) const;
 
       virtual iter_type 
       do_get(iter_type, iter_type, ios_base&, ios_base::iostate&, bool&) const;
@@ -994,7 +1056,8 @@ namespace std
     num_get<char, istreambuf_iterator<char> >::    
     _M_extract(istreambuf_iterator<char> __beg, 
 	       istreambuf_iterator<char> __end, ios_base& __io, 
-	       ios_base::iostate& __err, char* __xtrc, int& __base) const;
+	       ios_base::iostate& __err, char* __xtrc, 
+	       int& __base, bool __fp) const;
 
   // _Numeric_put is used by num_put, money_put, and time_put
   //   to help in formatting out numbers.
@@ -1441,13 +1504,13 @@ namespace std
       { return time_base::ymd; }
 
       virtual iter_type 
-      do_get_time(iter_type __s, iter_type __end, ios_base&,
-		  ios_base::iostate& __err, tm* __t) const
+      do_get_time(iter_type __s, iter_type /*__end*/, ios_base&,
+		  ios_base::iostate& /*__err*/, tm* /*__t*/) const
       { return __s; }
 
       virtual iter_type 
-      do_get_date(iter_type __s, iter_type __end, ios_base&,
-		  ios_base::iostate& __err, tm* __t) const
+      do_get_date(iter_type __s, iter_type /*__end*/, ios_base&,
+		  ios_base::iostate& /*__err*/, tm* /*__t*/) const
       { return __s; }
 
       virtual iter_type 
@@ -1459,8 +1522,8 @@ namespace std
 		       ios_base::iostate& __err, tm* __t) const;
 
       virtual iter_type 
-      do_get_year(iter_type __s, iter_type __end, ios_base&,
-		   ios_base::iostate& __err, tm* __t) const
+      do_get_year(iter_type __s, iter_type /*__end*/, ios_base&,
+		   ios_base::iostate& /*__err*/, tm* /*__t*/) const
       { return __s; }
 
       mutable basic_string<_CharT>* _M_daynames;
@@ -1494,8 +1557,9 @@ namespace std
 
       // NB: this is a nonvirtual, calls do_put in a loop.
       iter_type 
-      put(iter_type __s, ios_base& __f, char_type __fill, const tm* __tmb,
-	  const _CharT* __pattern, const _CharT* __pat_end) const
+      put(iter_type __s, ios_base& /*__f*/, char_type /*__fill*/,
+          const tm* /*__tmb*/, const _CharT* /*__pattern*/,
+          const _CharT* /*__pat_end*/) const
       { return __s; }
 
       iter_type 
@@ -1510,8 +1574,8 @@ namespace std
       ~time_put() { }
 
       virtual iter_type 
-      do_put(iter_type __s, ios_base&, char_type, const tm* __t, 
-	     char __format, char __mod) const
+      do_put(iter_type __s, ios_base&, char_type, const tm* /*__t*/, 
+	     char /*__format*/, char /*__mod*/) const
       { return __s; }
     };
 
@@ -1557,13 +1621,15 @@ namespace std
       ~money_get() { }
 
       virtual iter_type 
-      do_get(iter_type __s, iter_type __end, bool __intl, ios_base& __io,
-	     ios_base::iostate& __err, long double& __units) const
+      do_get(iter_type __s, iter_type /*__end*/, bool /*__intl*/,
+             ios_base& /*__io*/, ios_base::iostate& /*__err*/,
+             long double& /*__units*/) const
       { return __s; }
 
       virtual iter_type 
-      do_get(iter_type __s, iter_type __end, bool __intl, ios_base& __io,
-	     ios_base::iostate& __err, string_type& __digits) const
+      do_get(iter_type __s, iter_type /*__end*/, bool /*__intl*/,
+             ios_base& /*__io*/, ios_base::iostate& /*__err*/,
+             string_type& /*__digits*/) const
       { return __s; }
     };
 
@@ -1595,13 +1661,13 @@ namespace std
       ~money_put() { }
 
       virtual iter_type
-      do_put(iter_type __s, bool, ios_base& __io, char_type __fill,
-	     long double __units) const
+      do_put(iter_type __s, bool, ios_base& /*__io*/, char_type /*__fill*/,
+	     long double /*__units*/) const
       { return __s; }
 
       virtual iter_type
-      do_put(iter_type __s, bool, ios_base& __io, char_type __fill,
-	     const string_type& __digits) const
+      do_put(iter_type __s, bool, ios_base& /*__io*/, char_type /*__fill*/,
+	     const string_type& /*__digits*/) const
       { return __s; }
     };
 
@@ -1768,7 +1834,7 @@ namespace std
       { return 0; }
 
       virtual string_type  
-      do_get(catalog, int, int __msgid, const string_type& __dfault) const
+      do_get(catalog, int, int /*__msgid*/, const string_type& __dfault) const
       { return __dfault; }
 
       virtual void    
