@@ -65,27 +65,16 @@ namespace std {
     public:
       // 27.6.2.2 Constructor/destructor:
       explicit 
-      basic_ostream(__streambuf_type* __sb) : __ios_type(__sb)  
+      basic_ostream(__streambuf_type* __sb)
       { 
 	// XXX Need to fix for imbue events, as currently _M_fnumput
 	// is not updated on pubimbue calls. Need a callback? Needs to
 	// be moved down do basic_ios??
+	this->init(__sb);
 	_M_fnumput =  &use_facet<__numput_type>(this->getloc());
       }
 
-      // Needed for [io]stringstream/[io]fstream classes. This
-      // assumes that basic_ios has already been initialized
-      // _M_streambuf with some relevant buffer.
-      explicit 
-      basic_ostream() 
-      { 
- 	// XXX Need to fix for imbue events, as currently _M_fnumput
-	// is not updated on pubimbue calls. Need a callback? Needs to
- 	// be moved down do basic_ios??
- 	_M_fnumput =  &use_facet<__numput_type>(this->getloc());
-      }
-
-     virtual 
+      virtual 
       ~basic_ostream() 
       { _M_fnumput = NULL; }
 
