@@ -45,49 +45,57 @@ namespace std {
     // fill plain array __a[<__n>] with __t
     template<typename _Tp>
     inline void
-    __valarray_fill (_Tp* __a, size_t __n, const _Tp& __t)
+    __valarray_fill (_Tp* __restrict__ __a, size_t __n, const _Tp& __t)
     { while (__n--) *__a++ = __t; }
 
     // fill strided array __a[<__n-1 : __s>] with __t
     template<typename _Tp>
     inline void
-    __valarray_fill (_Tp* __a, size_t __n, size_t __s, const _Tp& __t)
+    __valarray_fill (_Tp* __restrict__ __a, size_t __n,
+                     size_t __s, const _Tp& __t)
     { for (size_t __i=0; __i<__n; ++__i, __a+=__s) *__a = __t; }
 
     // fill indirect array __a[__i[<__n>]] with __i
     template<typename _Tp>
     inline void
-    __valarray_fill(_Tp* __a, const size_t* __i, size_t __n, const _Tp& __t)
+    __valarray_fill(_Tp* __restrict__ __a, const size_t* __restrict__ __i,
+                    size_t __n, const _Tp& __t)
     { for (size_t __j=0; __j<__n; ++__j, ++__i) __a[*__i] = __t; }
     
     // copy plain array __a[<__n>] in __b[<__n>]
     template<typename _Tp>
     inline void
-    __valarray_copy (const _Tp* __a, size_t __n, _Tp* __b)
+    __valarray_copy (const _Tp* __restrict__ __a, size_t __n,
+                     _Tp* __restrict__ __b)
     { memcpy (__b, __a, __n * sizeof(_Tp)); }
 
     // copy strided array __a[<__n : __s>] in plain __b[<__n>]
     template<typename _Tp>
     inline void
-    __valarray_copy (const _Tp* __a, size_t __n, size_t __s, _Tp* __b)
+    __valarray_copy (const _Tp* __restrict__ __a, size_t __n, size_t __s,
+                     _Tp* __restrict__ __b)
     { for (size_t __i=0; __i<__n; ++__i, ++__b, __a += __s) *__b += *__a; }
 
     // copy plain __a[<__n>] in strided __b[<__n : __s>]
     template<typename _Tp>
     inline void
-    __valarray_copy (const _Tp* __a, _Tp* __b, size_t __n, size_t __s)
+    __valarray_copy (const _Tp* __restrict__ __a, _Tp* __restrict__ __b,
+                     size_t __n, size_t __s)
     { for (size_t __i=0; __i<__n; ++__i, ++__a, __b+=__s) *__b = *__a; }
 
     // copy indexed __a[__i[<__n>]] in plain __b[<__n>]
     template<typename _Tp>
     inline void
-    __valarray_copy (const _Tp* __a, const size_t* __i, _Tp* __b, size_t __n)
+    __valarray_copy (const _Tp* __restrict__ __a,
+                     const size_t* __restrict__ __i,
+                     _Tp* __restrict__ __b, size_t __n)
     { for (size_t __j=0; __j<__n; ++__j, ++__b, ++__i) *__b = __a[*__i]; }
 
     // copy plain __a[<__n>] in indexed __b[__i[<__n>]]
     template<typename _Tp>
     inline void
-    __valarray_copy (const _Tp* __a, size_t __n, _Tp* __b, const size_t* __i)
+    __valarray_copy (const _Tp* __restrict__ __a, size_t __n,
+                     _Tp* __restrict__ __b, const size_t* __restrict__ __i)
     { for (size_t __j=0; __j<__n; ++__j, ++__a, ++__i) __b[*__i] = *__a; }
 
     //
@@ -99,14 +107,14 @@ namespace std {
     template<typename _Tp> struct _Array {
         
         explicit _Array (size_t);
-        explicit _Array (_Tp* const);
+        explicit _Array (_Tp* const __restrict__);
         explicit _Array (const valarray<_Tp>&);
-        _Array (const _Tp*, size_t);
+        _Array (const _Tp* __restrict__, size_t);
         
         void free_data() const;
         _Tp* begin () const;
         
-        _Tp* const _M_data;
+        _Tp* const __restrict__ _M_data;
     };
 
     template<typename _Tp>
@@ -153,17 +161,20 @@ namespace std {
     { __valarray_copy (__a._M_data, __n, __b._M_data, __i._M_data); }
 
     template<typename _Tp>
-    inline _Array<_Tp>::_Array (size_t __n) : _M_data (new _Tp[__n]) {}
+    inline
+    _Array<_Tp>::_Array (size_t __n) : _M_data (new _Tp[__n]) {}
 
     template<typename _Tp>
-    inline _Array<_Tp>::_Array (_Tp* const __p) : _M_data (__p) {}
+    inline
+    _Array<_Tp>::_Array (_Tp* const __restrict__ __p) : _M_data (__p) {}
 
     template<typename _Tp>
     inline _Array<_Tp>::_Array (const valarray<_Tp>& __v) 
             : _M_data (__v._M_data) {}
 
     template<typename _Tp>
-    inline _Array<_Tp>::_Array (const _Tp* __b, size_t __s) 
+    inline
+    _Array<_Tp>::_Array (const _Tp* __restrict__ __b, size_t __s) 
             : _M_data (new _Tp[__s]) { __valarray_copy (__b, __s, _M_data); }
 
     template<typename _Tp>
@@ -324,7 +335,7 @@ _DEFINE_ARRAY_FUNCTION(>>, shift_right)
 
 } // std::
 
-#ifdef _G_NO_TEMPLATE_EXPORT
+#ifdef _GLIBCPP_NO_TEMPLATE_EXPORT
 # define export 
 # include <bits/valarray_array.tcc>    
 #endif
