@@ -27,12 +27,13 @@
 #ifndef _CPP_VECTOR
 #define _CPP_VECTOR 1
 
+#include <bits/stl_range_errors.h>
 #include <bits/stl_algobase.h>
 #include <bits/stl_alloc.h>
 #include <bits/stl_construct.h>
 #include <bits/stl_uninitialized.h>
 #include <bits/stl_vector.h>
-#include <bits/stl_bvector.h>
+//#include <bits/stl_bvector.h> // 990503 bkoz, non-standard
 
 #endif /* _CPP_VECTOR */
 
