@@ -355,7 +355,7 @@ public:
   template<class _Integer>
   void _M_insert_dispatch(iterator __pos, _Integer __n, _Integer __x,
                           __true_type) {
-    insert(__pos, (size_type) __n, (_Tp) __x);
+    _M_fill_insert(__pos, (size_type) __n, (_Tp) __x);
   }
 
   template <class _InputIterator>
@@ -374,8 +374,10 @@ public:
   void insert(iterator __position,
               const_iterator __first, const_iterator __last);
 #endif /* __STL_MEMBER_TEMPLATES */
-  void insert(iterator __pos, size_type __n, const _Tp& __x);
- 
+  void insert(iterator __pos, size_type __n, const _Tp& __x)
+    { _M_fill_insert(__pos, __n, __x); }
+  void _M_fill_insert(iterator __pos, size_type __n, const _Tp& __x); 
+
   void push_front(const _Tp& __x) { insert(begin(), __x); }
   void push_front() {insert(begin());}
   void push_back(const _Tp& __x) { insert(end(), __x); }
@@ -444,7 +446,9 @@ public:
   // The range version is a member template, so we dispatch on whether
   // or not the type is an integer.
 
-  void assign(size_type __n, const _Tp& __val);
+  void assign(size_type __n, const _Tp& __val) { _M_fill_assign(__n, __val); }
+
+  void _M_fill_assign(size_type __n, const _Tp& __val);
 
 #ifdef __STL_MEMBER_TEMPLATES
 
@@ -456,7 +460,7 @@ public:
 
   template <class _Integer>
   void _M_assign_dispatch(_Integer __n, _Integer __val, __true_type)
-    { assign((size_type) __n, (_Tp) __val); }
+    { _M_fill_assign((size_type) __n, (_Tp) __val); }
 
   template <class _InputIterator>
   void _M_assign_dispatch(_InputIterator __first, _InputIterator __last,
@@ -605,7 +609,8 @@ list<_Tp, _Alloc>::insert(iterator __position,
 
 template <class _Tp, class _Alloc>
 void 
-list<_Tp, _Alloc>::insert(iterator __position, size_type __n, const _Tp& __x)
+list<_Tp, _Alloc>::_M_fill_insert(iterator __position,
+                                  size_type __n, const _Tp& __x)
 {
   for ( ; __n > 0; --__n)
     insert(__position, __x);
@@ -652,7 +657,7 @@ list<_Tp, _Alloc>& list<_Tp, _Alloc>::operator=(const list<_Tp, _Alloc>& __x)
 }
 
 template <class _Tp, class _Alloc>
-void list<_Tp, _Alloc>::assign(size_type __n, const _Tp& __val) {
+void list<_Tp, _Alloc>::_M_fill_assign(size_type __n, const _Tp& __val) {
   iterator __i = begin();
   for ( ; __i != end() && __n > 0; ++__i, --__n)
     *__i = __val;
