@@ -20,7 +20,7 @@
  * provided that the above copyright notice appear in all copies and
  * that both that copyright notice and this permission notice appear
  * in supporting documentation.  Silicon Graphics makes no
- * representations about the suitability of this software for any
+ * representations about the suitability of this  software for any
  * purpose.  It is provided "as is" without express or implied warranty.
  */
 
@@ -556,7 +556,7 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
     }
     else if (size() >= __xlen) {
       iterator __i(copy(__x.begin(), __x.end(), begin()));
-      destroy(__i, _M_finish);
+      destroy(__i.base(), _M_finish);
     }
     else {
       copy(__x.begin(), __x.begin() + size(), _M_start);
@@ -612,7 +612,7 @@ vector<_Tp, _Alloc>::_M_assign_aux(_ForwardIter __first, _ForwardIter __last,
   }
   else if (size() >= __len) {
     iterator __new_finish(copy(__first, __last, _M_start));
-    destroy(__new_finish, _M_finish);
+    destroy(__new_finish.base(), _M_finish);
     _M_finish = __new_finish;
   }
   else {
@@ -666,7 +666,8 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
   if (_M_finish != _M_end_of_storage) {
     construct(_M_finish, *(_M_finish - 1));
     ++_M_finish;
-    copy_backward(__position, _M_finish - 2, _M_finish - 1);
+    copy_backward(__position, iterator(_M_finish - 2), 
+		  iterator(_M_finish - 1));
     *__position = _Tp();
   }
   else {
@@ -675,10 +676,12 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
     pointer __new_start = _M_allocate(__len);
     pointer __new_finish = __new_start;
     __STL_TRY {
-      __new_finish = uninitialized_copy(_M_start, __position, __new_start);
+      __new_finish = uninitialized_copy(iterator(_M_start), __position, 
+					__new_start);
       construct(__new_finish);
       ++__new_finish;
-      __new_finish = uninitialized_copy(__position, _M_finish, __new_finish);
+      __new_finish = uninitialized_copy(__position, iterator(_M_finish), 
+					__new_finish);
     }
     __STL_UNWIND((destroy(__new_start,__new_finish), 
                   _M_deallocate(__new_start,__len)));
