@@ -26,19 +26,17 @@
 #include <assert.h>
 #endif
 
-using namespace std;
-
 int test01(void)
 {
   bool test = true;
 
-  string str1;
-  string str2;
+  std::string str1;
+  std::string str2;
   
   // Should get this:
   // 1:8-chars_8-chars_
   // 2:8-chars_8-chars_
-  str1 = string("8-chars_") + "8-chars_";
+  str1 = std::string("8-chars_") + "8-chars_";
   const char* p1 = str1.c_str();
   // printf("1:%s\n", str1.c_str());
   str2 = str1 + "7-chars";
