@@ -34,6 +34,7 @@ struct B { };
 bool test01()
 {
 
+  // 1
   bool test = true;
   std::vector< A<B> > vec01;
   std::vector< A<B> > vec02(5);
@@ -48,6 +49,10 @@ bool test01()
   return test;
 }
 
+// 2
+template class vector<double>;
+template class vector< A<B> >;
+
 int main()
 {
   test01();
